from.bmp_14_tillage_management import TillageManagement

class Pasture_tillage_management(TillageManagement):
    """Distribution Table for BMP: Pasture tillage management (36)"""
    def __init__(self, TillMon, TillDay, TillCode):
        super().__init__(TillMon, TillDay, TillCode)
