class StructureAttribute:
    def __init__(self, id:int, area_ha:float, subbasin:int, contribution_area_ha:float):
        self.id = id
        self.area_ha = area_ha
        self.subbasin = subbasin
        self.contribution_area = contribution_area_ha