from .bmp_management_base import BMPManagementBase

class ManureSpringApplicationRatherThanFallApplicationManagement(BMPManagementBase):
    """Distribution Table for BMP: Spring application rather than fall application (24)"""
    def __init__(self):
        super().__init__()
