from .bmp_management_base import BMPManagementBase

class ManureNoApplicaitonOnSnowManagement(BMPManagementBase):
    """Distribution Table for BMP: No application on snow (23)"""
    def __init__(self):
        super().__init__()
